import { Search } from '../search.mjs';

export class JumpSearch extends Search {
  /**
   * @param {*[]} sortedArray - sorted array to search in
   * @param {*} wantedElement - element to search in the sorted array
   * @return {Number} - zero based index in the array if element is found, otherwise -1
   */
  search(sortedArray, wantedElement) {
    const size = sortedArray.length;
    const step = Math.floor(Math.sqrt(size)); // jump size

    // Can't find anything in empty array
    if (!size) return -1;

    // start and end indexes of the "block"
    // that eventually contains the "wantedElement"
    let prevIndex = 0;
    let nextIndex = step;

    while (this.comparator.lt(sortedArray[Math.min(nextIndex, size) - 1], wantedElement)) {
      // jump to next block
      prevIndex = nextIndex;
      nextIndex += step;

      // if next block is out of the array, then wantedElement is not in this array
      if (prevIndex >= size) {
        return -1;
      }
    }

    // Do linear search for the "wantedElement" in the last "block" subarray
    while (this.comparator.lt(sortedArray[prevIndex], wantedElement)) {
      prevIndex += 1;
      if (prevIndex === Math.min(nextIndex, size)) {
        return -1;
      }
    }

    if (this.comparator.eq(sortedArray[prevIndex], wantedElement)) {
      return prevIndex;
    }

    return -1;
  }
}
