import { Comparator } from '../../comparator/comparator.mjs';

export class Search {
  constructor(originalCallbacks) {
    this.callbacks = Search.initSearchCallbacks(originalCallbacks);
    this.comparator = new Comparator(this.callbacks.compareCallback);
  }

  /**
   * @param {SearchCallbacks} originalCallbacks
   * @return {SearchCallbacks}
   */
  static initSearchCallbacks(originalCallbacks) {
    const callbacks = originalCallbacks || {};
    const stubCallback = () => {};

    callbacks.compareCallback = callbacks.compareCallback || undefined;
    callbacks.visitingCallback = callbacks.visitingCallback || stubCallback;

    return callbacks;
  }

  /**
   * Derived classes should implement search method
   */
  search() {
    throw new Error('Not implemented');
  }

  /**
   * Derived classes should implement searchAll method
   */
  searchAll() {
    throw new Error('Not implemented');
  }
}
