import { Sort } from '../sort.mjs';

export class CombSort extends Sort {
  /**
   * @param {Object} originalCallbacks
   * @param {Number} factor
   */
  constructor(originalCallbacks, factor = 10/13) {
    super(originalCallbacks);
    this.factor = factor;
  }

  /**
   * Find next gap from current one
   *
   * @param {Number} gap - current gap
   * @return {Number} newGap - new gap
   */
  getNextGap(gap) {
    // shrink gap by shrink factor
    const newGap = gap * this.factor;
    return newGap > 1 ? newGap : 1;
  }

  /**
   * stable
   * sort in place
   *
   * @param {*[]} array - array to sort
   * @return {*[]} array - sorted array
   */
  sort(array) {
    const len = array.length;

    let gap = len;
    let swapped = true;

    while(gap !== 1 || swapped === true) {
      swapped = false;
      gap = this.getNextGap(gap);

      for (let i = 0; i < len - gap; i++) {
        const idx = Math.floor(i + gap);

        //* call visiting callback *//
        this.callbacks.visitingCallback(array[i], array[idx]);

        if (this.comparator.gt(array[i], array[idx])) {
          [array[i], array[idx]] = [array[idx], array[i]];
          swapped = true;
        }
      }
    }

    return array;
  }
}
