import { Sort } from '../sort.mjs';

export class CountingSort extends Sort {
  /**
   * sort out-of-place
   *
   * @param {Number[]} originalArray
   * @return {Number[]} sortedArray
   */
  sort(originalArray = []) {
    if (!originalArray.length) return [];

    /* We need min and max elements in order to build the number bucket array later */
    const min = Math.min(...originalArray);
    const max = Math.max(...originalArray);

    /**
     * Init buckets array.
     * This array will hold frequency of each number from originalArray.
     */
    const buckets = Array(max - min + 1).fill(0);

    /* Count frequency of each number */
    originalArray.forEach(element => buckets[element - min]++);

    /**
     * Add previous frequencies to the current one for each number in bucket
     * to detect how many numbers less then current one should be standing to
     * the left of current one.
     */
    for (let i = 1; i < buckets.length; i++) {
      buckets[i] += buckets[i - 1];
    }

    /**
     * Now let's shift frequencies to the right, so that they show correct numbers.
     * I.e. If we won't shift right than the value of buclets[5] will display how many
     * elements less than 5 should be placed to the left of 5 in sorted array
     * INCLUDING 5th. After shifting though this number will not include 5th anymore.
     */
    buckets.pop();
    buckets.unshift(0);

    /* Now lets assemble sorted array */
    const sortedArray = Array(originalArray.length).fill(null);

    originalArray.forEach((element, index) => {
      /* Get correct position of this element in sorted array. */
      const elementSortedIndex = buckets[element - min];

      /* Put element into correct position in sorted array. */
      sortedArray[elementSortedIndex] = element;

      /* Increase position of current element in the bucket for future correct placements. */
      buckets[element - min]++;
    });

    return sortedArray;
  }
}
