import { Sort } from '../sort.mjs';
import { MinHeap } from '../../../data-structures/index.mjs';

export class HeapSort extends Sort {
  /**
   * in-place, not-stable
   *
   * @param {Number[]} array
   * @return {Number[]} sorted array
   */
  sort(array) {
    const heap = new MinHeap(this.callbacks.compareCallback);

    //** Insert all array elements to the heap **//
    while (array.length) {
      heap.add(array.pop());
    }

    /**
     * Now we have min heap with minimal element always on top.
     * Let's poll that min element one by one and thus form the sorted array.
     */
    while (!heap.isEmpty()) {
      array.push(heap.poll());
    }

    return array;
  }
}
