import { Sort } from '../sort.mjs';
import { MergeSort } from './merge-sort.mjs';
import { Stack } from '../../../data-structures/index.mjs';

export class IterativeMergeSort extends Sort {
  /**
   * stable
   * in-place
   * non-recursive
   * iterative merge sort
   * bottom-up merge sort
   * requires extra space because of merge
   *
   * @param {*[]} array - array to sort
   * @return {*[]} array - sorted array
   */
  sort(array) {
    /* cache array length */
    const len = array.length;

    /* starts with width = 1, and then 2, 4, 8, 16 ... etc */
    for (let width = 1; width < len; width *= 2) {
      for (let leftIndex = 0; leftIndex < len - width; leftIndex += width * 2) {
        const middleIndex = leftIndex + width - 1;
        const rightIndex = Math.min(leftIndex + width + width - 1, len - 1);
        this.merge(array, leftIndex, middleIndex, rightIndex);
      }
    }

    return array;
  }

  /**
   * in place merge
   * assert sorted array [leftIndex, middleIndex]
   * assert sorted array [middleIndex + 1, rightIndex]
   *
   * @param {*[]} inputArray - The array to use for merging
   * @param {Number} leftIndex - Start index to use for merge
   * @param {Number} middleIndex - Middle index to use for merge
   * @param {NUmber} rightIndex - End index to use for merge
   * @return {*[]} inputArray - The merged result
   */
  merge(inputArray, leftIndex, middleIndex, rightIndex) {
    /* copy */
    const auxArray = [...inputArray];

    let i = leftIndex;
    let j = middleIndex + 1;

    /* merge */
    for (let k = leftIndex; k <= rightIndex; k++) {
      if (i > middleIndex) {
        inputArray[k] = auxArray[j++];
      }
      else if (j > rightIndex) {
        inputArray[k] = auxArray[i++];
      }
      else if (this.comparator.lt(auxArray[j], auxArray[i])) {
        inputArray[k] = auxArray[j++];
      }
      else {
        inputArray[k] = auxArray[i++];
      }
      /* call visiting callback */
      this.callbacks.visitingCallback(inputArray[k]);
    }

    return inputArray;
  }
}
