import { Sort } from '../sort.mjs';

export class MergeSort extends Sort {
  /**
   * stable
   * out-of-place
   * top-down merge sort
   * recursive merge sort
   *
   * @param {*[]} originalArray - array to sort
   * @return {*[]} sortedArray - new sorted array
   */
  sort(originalArray) {
    /* call visiting callback */
    this.callbacks.visitingCallback(null);

    /* Array with one element is considered sorted array */
    if (originalArray.length <= 1) {
      return originalArray;
    }

    /* Find middle index of the array */
    const middleIndex = Math.floor(originalArray.length / 2);

    /* Split the array in two halves */
    const leftArray = originalArray.slice(0, middleIndex);
    const rightArray = originalArray.slice(middleIndex, originalArray.length);

    /* Sort the two halves */
    const leftArraySorted = this.sort(leftArray);
    const rightArraySorted = this.sort(rightArray);

    /* Merge the two sorted arrays into one sorted array */
    return this.mergeSortedArrays(leftArraySorted, rightArraySorted);
  }

  /**
   * Merge two sorted arrays into one sorted array
   *
   * @param {*[]} leftArray - sorted array
   * @param {*[]} rightArray - sorted array
   * @return {*[]} resultArray - merged sorted array
   */
  mergeSortedArrays(leftArray, rightArray) {
    const resultArray = [];

    while (leftArray.length && rightArray.length) {
      const min = this.comparator.lte(leftArray[0], rightArray[0]) ? leftArray.shift() : rightArray.shift();
      resultArray.push(min);
      this.callbacks.visitingCallback(min);
    }

    if (leftArray.length)
      while (leftArray.length)
        resultArray.push(leftArray.shift());

    if (rightArray.length)
      while (rightArray.length)
        resultArray.push(rightArray.shift());

    return resultArray;
  }
}
