import { Sort } from '../sort.mjs';

export class PermutationSort extends Sort {
  /**
   * @param {*[]} array
   */
  sort(array) {
    while (!this.isSorted(array)) {
      this.shuffle(array);
    }
  }

  /**
   * Fisher–Yates shuffle
   *
   * @param {*[]} array
   */
  shuffle(array) {
    for (let i = array.length - 1; i > 0; i--) {
      const j = Math.floor(Math.random() * (i + 1));
      [array[i], array[j]] = [array[j], array[i]];
    }
  }

  /**
   * Check if array is sorted
   *
   * @param {*[]} array
   */
  isSorted(array) {
    for (let i = 0; i < array.length - 1; i++) {
      if (array[i] > array[i+1]) {
        return false;
      }
    }
    return true;
  }
}
