import { QuickSort } from './quick-sort.mjs';
import { Stack } from '../../../data-structures/index.mjs';

/**
 * We will override "sort" method from QuickSort class
 * We will reuse "partition" method from QuickSort class
 */
export class IterativeQuickSort extends QuickSort {
  /**
   * in-place
   * not-stable
   *
   * @param {*[]} array - original array
   * @param {Number} inputLowIndex - low index
   * @param {Number} inputHighIndex - high index
   * @return {*[]} array - sorted array
   */
  sort(array, inputLowIndex = 0, inputHighIndex = array.length - 1) {
    const len = array.length;

    // if array has less than or equal to one elements then it is already sorted
    if (len <= 1) return array;

    // Replace recursion with stack
    // Push low and high index to the stack
    const stack = new Stack();
    stack.push(inputLowIndex);
    stack.push(inputHighIndex);

    while (!stack.isEmpty()) {
      const highIndex = stack.pop();
      const lowIndex = stack.pop();
      const pivotIndex = this.partition(array, lowIndex, highIndex);

      if (pivotIndex - 1 > lowIndex) {
        stack.push(lowIndex);
        stack.push(pivotIndex - 1);
      }

      if (pivotIndex + 1 < highIndex) {
        stack.push(pivotIndex + 1);
        stack.push(highIndex);
      }
    }

    return array;
  }
}
