import { Sort } from '../sort.mjs';

export class QuickSort extends Sort {
  /**
   * Pick last element as "pivot" and put it in correct position (index) in the sorted array.
   * Places all smaller elements before pivot (to the left).
   * Places all greater elements after pivot (to the right).
   *
   * @param {*[]} array
   * @param {Number} lowIndex
   * @param {Number} highIndex
   * @return {Number} pivotIndex
   */
  partition(array, lowIndex = 0, highIndex = array.length - 1) {
    let index = lowIndex - 1;
    const pivot = array[highIndex];

    //** call visiting callback **//
    this.callbacks.visitingCallback(pivot);

    for (let j = lowIndex; j < highIndex; j++) {
      if (this.comparator.lt(array[j], pivot)) {
        index++;
        [array[index], array[j]] = [array[j], array[index]];
      }
    }

    [array[index + 1], array[highIndex]] = [array[highIndex], array[index + 1]];
    return index + 1;
  }

  /**
   * not-stable
   * sort-in-place
   *
   * @param {*[]} array
   * @param {Number} lowIndex
   * @param {Number} highIndex
   * @return {*[]} - sorted array
   */
  sort(array, lowIndex = 0, highIndex = array.length - 1) {
    if (lowIndex < highIndex) {
      //** final pivot index **//
      const pivotIndex = this.partition(array, lowIndex, highIndex);

      //** Separately sort elements before and after pivot **//
      this.sort(array, lowIndex, pivotIndex - 1);
      this.sort(array, pivotIndex + 1, highIndex);
    }
    return array;
  }
}
