import { Sort } from '../sort.mjs';

/*
 * 3-Way Quick Sort
 *
 * It's not stable with "pop"
 * It's stable with "shift", but way more slower
 */

export class ThreeWayQuickSort extends Sort {
  /**
   * stable || not-stable
   * in-place || out-of-place
   *
   * @param {*[]} originalArray
   * @param {Boolean} stable - whether to do stable sorting
   * @return {*[]} sortedArray - sorted array
   */
  sort(originalArray, stable = false) {
    const len = originalArray.length;

    // if array has less than or equal to one elements then it is already sorted
    if (len <= 1) return [...originalArray];

    // If we want stable sort we should pick the first element
    const pivotElement = stable ? originalArray.shift() : originalArray.pop();

    // We need 3 arrays for "lt", "eq", "gt" elements of pivot
    const leftArray = [];
    const rightArray = [];
    const centerArray = [pivotElement];

    while (originalArray.length) {
      const currentElement = stable ? originalArray.shift() : originalArray.pop();

      // call visiting callback
      this.callbacks.visitingCallback(currentElement);

      if (this.comparator.eq(currentElement, pivotElement)) {
        centerArray.push(currentElement);
      }
      else if (this.comparator.lt(currentElement, pivotElement)) {
        leftArray.push(currentElement);
      }
      else {
        rightArray.push(currentElement);
      }
    }

    // Sort left and right array
    const leftArraySorted = this.sort(leftArray);
    const rightArraySorted = this.sort(rightArray);

    // Join all arrays in relative order to get the result
    const sortedArray = [...leftArraySorted, ...centerArray, ...rightArraySorted];
    return sortedArray;
  }
}
