import { Sort } from '../sort.mjs';

export class ShellSort extends Sort {
  /**
   * not stable
   * sort in place
   *
   * @param {*[]} array - array to sort
   * @return {*[]} array - sorted array
   */
  sort(array) {
    const len = array.length;

    //** Define a gap distance **//
    let gap = Math.floor(len / 2);

    //** Until gap is bigger then zero do elements comparisons and swaps **//
    while (gap > 0) {

      //** Go and compare all distant element pairs **//
      for (let i = 0; i < len - gap; i++) {
        let idx = i; //** Current Index **//
        let gapIdx = i + gap; //** Gap Shifted Index **//

        while (idx >= 0) {
          //** Call visiting callback **//
          this.callbacks.visitingCallback(array[idx]);

          //** Compare and swap elements if needed **//
          if (this.comparator.lt(array[gapIdx], array[idx])) {
            [array[idx], array[gapIdx]] = [array[gapIdx], array[idx]]
          }
          gapIdx = idx;
          idx -= gap;
        }
      }

      //** Shrink the gap **//
      gap = Math.floor(gap / 2);
    }

    return array;
  }
}
