import test from 'ava';
import sinon from 'sinon';

export const EQUAL_ARRAY = [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1];
export const SORTED_ARRAY = [0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9];
export const SHUFFLED_ARRAY = [7, 5, 8, 2, 9, 1, 0, 1, 3, 6, 4, 4, 6, 7, 8, 9, 2, 3, 5, 0];
export const REVERSE_SORTED_ARRAY = [9, 9, 8, 8, 7, 7, 6, 6, 5, 5, 4, 4, 3, 3, 2, 2, 1, 1, 0, 0];
export const NEGATIVE_ARRAY = [-1, -2, -3, -10, ...SHUFFLED_ARRAY];
export const SORTED_NEGATIVE_ARRAY = [-10, -3, -2, -1, ...SORTED_ARRAY];
export const REVERSE_SORTED_NEGATIVE_ARRAY = [...REVERSE_SORTED_ARRAY, -1, -2, -3, -10]

let counter = 0;

export class TestSorter {
  static testSorting(SortingClass) {
    const sorter = new SortingClass();
    const Type = SortingClass.name;

    test(`${Type} should sort array`, expect => {
      expect.deepEqual(sorter.sort([]), []);
      expect.deepEqual(sorter.sort([1]), [1]);
      expect.deepEqual(sorter.sort([1, 2]), [1, 2]);
      expect.deepEqual(sorter.sort([4, 2, 1]), [1, 2, 4]);
      expect.deepEqual(sorter.sort([...EQUAL_ARRAY]), EQUAL_ARRAY);
      expect.deepEqual(sorter.sort([...SORTED_ARRAY]), SORTED_ARRAY);
      expect.deepEqual(sorter.sort([...SHUFFLED_ARRAY]), SORTED_ARRAY);
      expect.deepEqual(sorter.sort([...REVERSE_SORTED_ARRAY]), SORTED_ARRAY);
    });
  }

  static testNegativeSorting(SortingClass) {
    const sorter = new SortingClass();
    const Type = SortingClass.name;

    test(`${Type} should sort array containing negative numbers`, expect => {
      expect.deepEqual(sorter.sort([]), []);
      expect.deepEqual(sorter.sort([-1]), [-1]);
      expect.deepEqual(sorter.sort([-1, 0, -2]), [-2, -1, 0]);
      expect.deepEqual(sorter.sort([-2, -4, 0]), [-4, -2, 0]);
      expect.deepEqual(sorter.sort([...NEGATIVE_ARRAY]), SORTED_NEGATIVE_ARRAY);
      expect.deepEqual(sorter.sort([...SORTED_NEGATIVE_ARRAY]), SORTED_NEGATIVE_ARRAY);
      expect.deepEqual(sorter.sort([...REVERSE_SORTED_NEGATIVE_ARRAY]), SORTED_NEGATIVE_ARRAY);
    });
  }

  static testCustomComparatorSorting(SortingClass) {
    const callbacks = {
      compareCallback: (a, b) => {
        if (a.length === b.length) return 0;
        return a.length < b.length ? -1 : 1;
      }
    };

    const sorter = new SortingClass(callbacks);
    const Type = SortingClass.name;

    test(`${Type} should sort array with custom comparator`, expect => {
      expect.deepEqual(sorter.sort(['']), ['']);
      expect.deepEqual(sorter.sort(['a']), ['a']);
      expect.deepEqual(sorter.sort(['aa', 'a']), ['a', 'aa']);
      expect.deepEqual(sorter.sort(['aa', 'q', 'bbbb', 'ccc']), ['q', 'aa', 'ccc', 'bbbb']);
      expect.deepEqual(sorter.sort(['aa', 'aa']), ['aa', 'aa']);
    });
  }

  static testSortingStability(SortingClass) {
    const callbacks = {
      compareCallback: (a, b) => {
        if (a.length === b.length) return 0;
        return a.length < b.length ? -1 : 1;
      }
    };

    const sorter = new SortingClass(callbacks);
    const Type = SortingClass.name;

    test(`${Type} should do stable sorting`, expect => {
      expect.deepEqual(sorter.sort(['bb', 'aa', 'c']), ['c', 'bb', 'aa']);
      expect.deepEqual(sorter.sort(['aa', 'x', 'a', 'bbbb', 'ccc']), ['x', 'a', 'aa', 'ccc', 'bbbb']);
    });
  }

  static testAlgorithmTimeComplexity(SortingClass, arrayToBeSorted, numberOfVisits) {
    const Type = SortingClass.name;
    const visitingCallback = sinon.fake();
    const callbacks = { visitingCallback }
    const sorter = new SortingClass(callbacks);

    counter++;

    test(`${Type} AC-${counter} should visit array element specified number of times`, expect => {
      sorter.sort([...arrayToBeSorted]);
      expect.is(visitingCallback.callCount, numberOfVisits);
    });
  }
}
