export class Comparator {
  /**
   * @param {function(a: *, b: *)} [compareFunction]
   */
  constructor(compareFunction) {
    this.compare = compareFunction || Comparator.defaultCompareFunction;
  }

  /**
   * @param {(string|number)} a
   * @param {{string|number}} b
   * @return {number}
   */
  static defaultCompareFunction(a, b) {
    if (a === b) {
      return 0;
    }
    return a < b ? -1 : 1;
  }

  /**
   * equal
   */
  eq(a, b) {
    return this.compare(a, b) === 0;
  }

  /**
   * less_than
   */
  lt(a, b) {
    return this.compare(a, b) < 0;
  }

  /**
   * greater_than
   */
  gt(a, b) {
    return this.compare(a, b) > 0;
  }

  /**
   * less_than_or_equal
   */
  lte(a, b) {
    return this.lt(a, b) || this.eq(a, b);
  }

  /**
   * greater_than_or_equal
   */
  gte(a, b) {
    return this.gt(a, b) || this.eq(a, b);
  }

  /**
   *
   */
  reverse() {
    const compareOriginal = this.compare;
    this.compare = (a, b) => compareOriginal(b, a);
  }
}
