import { LinkedList } from '../linked-list/linked-list.mjs';

export class Queue {
  /**
   * We are going to implement Queue based on LinkedList, since the two DS are quite similar.
   * They both operate mostly on the elements at the beginning and the end.
   */
  constructor() {
    this._list = new LinkedList();
  }

  /**
   * Add a new element to the end of the queue (the tail of the linked list).
   * This element will be processed after all elements ahead of it.
   * @param {*} value
   */
  enqueue(value) {
    this._list.append(value);
  }

  /**
   * Remove the element at the front of the queue (the head of the linked list).
   * If the queue is empty, return null;
   * @return {*}
   */
  dequeue() {
    const removedHead = this._list.deleteHead();
    return removedHead ? removedHead.value : null;
  }

  /**
   * Read the element at the front of the queue without removing it
   * @return {*}
   */
  peek() {
    if (this.isEmpty()) {
      return null;
    }
    return this._list.head.value;
  }

  /**
   * @return {Boolean}
   */
  isEmpty() {
    return !this._list.head;
  }

  /**
   * @return {*[]}
   */
  toArray() {
    return this._list.toArray().map(node => node.value);
  }

  /**
   * @param {function} [callback]
   * @return {String}
   */
  toString(callback) {
    return this._list.toString(callback);
  }
}
