/**
 * We are going to implement Stack based on LinkedList
 */

import { LinkedList } from '../linked-list/linked-list.mjs';

export class Stack {
  constructor() {
    this.list = new LinkedList();
  }

  /**
   * @param {*} value
   */
  push(value) {
    // just add the new value at the start of the linked list
    this.list.prepend(value);
  }

  /**
   * @return LinkedListNode value
   */
  pop() {
    // try to delete the head of linked list
    // if there is no head (the linked list is empty) just retur null
    const removed = this.list.deleteHead();
    return removed ? removed.value : null;
  }

  /**
   * @return {*}
   */
  peek() {
    if (this.isEmpty()) {
      return null;
    }

    // Read value from the start of linked list without deleting it
    return this.list.head.value;
  }

  /**
   * @return {Boolean}
   */
  isEmpty() {
    // The stack is empty if its linked list does not have a head
    return !this.list.head;
  }

  /**
   * @return {*[]}
   */
  toArray() {
    return this.list.toArray().map(node => node.value);
  }

  /**
   * @param {function} [callback]
   * @param {string}
   */
  toString(callback) {
    return this.list.toString(callback);
  }
}
