# Algorithms and Data Structures

## Big O

### Big Oh `O`

Big Oh is often used to describe the worst-case of an algorithm by taking the highest order of a polynomial function and ignoring all the constants value since they aren’t too influential for sufficiently large input.

### Big Omega `Ω`

Big Omega is the opposite of Big Oh, if Big Oh was used to describe the upper bound (worst-case) of a asymptotic function, Big Omega is used to describe the lower bound of a asymptotic function. In analysis algorithm, this notation is usually used to describe the complexity of an algorithm in the best-case, which means the algorithm will not be better than its best-case.

### Big Theta `Θ`

When an algorithm has a complexity with lower bound = upper bound, say that an algorithm has a complexity  `O(n*log(n))` and `Ω(n*log(n))`, it’s actually has the complexity `Θ(n*log(n))`, which means the running time of that algorithm always falls in `n*log(n)` in the best-case and worst-case.



## Data Structures

A data structure is a particular way of organizing and storing data in a computer so that it can be accessed and modified efficiently. More precisely, a data structure is a collection of data values, the relationships among them, and the functions or operations that can be applied to the data.

* [Linked List](./js/data-structures/linked-list)
* [Stack](./js/data-structures/queue)
* [Queue](./js/data-structures/queue)
* [Heap](./js/data-structures/heap)


## Algorithms

An algorithm is an unambiguous specification of how to solve a class of problems. It is a set of rules that precisely define a sequence of operations.


### Sorting

* [Insertion Sort](./js/algorithms/sorting/insertion-sort)
* [Shell Sort](./js/algorithms/sorting/shell-sort)
* [Bubble Sort](./js/algorithms/sorting/bubble-sort)
* [Shaker Sort](./js/algorithms/sorting/shaker-sort)
* [Comb Sort](./js/algorithms/sorting/comb-sort)
* [Selection Sort](./js/algorithms/sorting/selection-sort)
* [Heap Sort](./js/algorithms/sorting/heap-sort)
* [Quick Sort](./js/algorithms/sorting/quick-sort)
* [ThreeWayQuick Sort](./js/algorithms/sorting/quick-sort)
* [IterativeQuick Sort](./js/algorithms/sorting/quick-sort)
* [Merge Sort](./js/algorithms/sorting/merge-sort)
* [IterativeMerge Sort](./js/algorithms/sorting/merge-sort)
* [Tim Sort](./js/algorithms/sorting/tim-sort)
* [Counting Sort](./js/algorithms/sorting/counting-sort)
* [PositiveCounting Sort](./js/algorithms/sorting/counting-sort)
* [Bit Sort](./js/algorithms/sorting/bit-sort)
* [Radix Sort](./js/algorithms/sorting/radix-sort)
* [Permutation Sort](./js/algorithms/sorting/permutation-sort)


### Searching

* [Linear Search](./js/algorithms/searching/linear-search)
* [Jump Search](./js/algorithms/searching/jump-search)
* [Binary Search](./js/algorithms/searching/binary-search)
* ...


### References

* [big O cheatsheet](http://bigocheatsheet.com)
* [google python style guide](https://github.com/google/styleguide/blob/gh-pages/pyguide.md)
* [washington edu dsa course](https://courses.cs.washington.edu/courses/cse373/15su/lectures)
* [searching and sorting algorithms](https://codeburst.io/algorithms-i-searching-and-sorting-algorithms-56497dbaef20)
* [top algorithms and data structures you really need to know](https://towardsdatascience.com/top-algorithms-and-data-structures-you-really-need-to-know-ab9a2a91c7b5)
* [itsy bitsy data structures js](https://github.com/jamiebuilds/itsy-bitsy-data-structures/blob/master/itsy-bitsy-data-structures.js)
